-module(myapp_events).
-behaviour(gen_server).

-export([start_link/0]).
-export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).

-export([add_task/4, delete_task/2]).


-define(IS_STRING(S), (is_list(S) andalso S /= [] andalso is_integer(hd(S)))).


add_task(Task_Id, Proc_Id, Type, Time)
when ?IS_STRING(Task_Id), is_integer(Proc_Id), ((Type =:= static) orelse (Type =:= dynamic)), is_integer(Time) ->
  Time2 = case Type of
    dynamic ->
      erlang:system_time(second) + Time; % Timestamp_Now + Time
    _ ->
      % static
      Time
  end,
  gen_server:call(?MODULE, {add_task, {Task_Id, Proc_Id, Time2}}).

delete_task(Task_Id, Proc_Id)
when ?IS_STRING(Task_Id), is_integer(Proc_Id) ->
  gen_server:call(?MODULE, {delete_task, {Task_Id, Proc_Id}}).


start_link() ->
  gen_server:start_link({local, ?MODULE}, ?MODULE, [], []).


init([]) ->
  erlang:send_after(100, self(), check_time_process),
  State = dict:new(),
  {ok, State}.


handle_call({add_task, {Task_Id, Proc_Id, Time}}, _From, State) ->
  case dict:is_key({Task_Id, Proc_Id}, State) of
    true -> io:format("update: ~p ~p~n", [Task_Id, Proc_Id]);
    _ -> ok
  end,
  
  State2 = dict:store({Task_Id, Proc_Id}, Time, State),
  {reply, ok, State2};


handle_call({delete_task, {Task_Id, Proc_Id}=K}, _From, State) ->
  case dict:is_key(K, State) of
    true -> io:format("deleted: ~p ~p~n", [Task_Id, Proc_Id]);
    _ -> ok
  end,
  
  State2 = dict:erase(K, State),
  {reply, ok, State2};


handle_call(_Req, _From, State) ->
  {reply, not_handled, State}.


handle_cast(_Req, State) ->
  {noreply, State}.


handle_info(check_time_process, State) ->
  erlang:send_after(1000, self(), check_time_process),
  
  Timestamp_Now = erlang:system_time(second),
  State2 = dict:filter(fun({Task_Id0, Proc_Id0}, Time0) ->
    if Time0 =< Timestamp_Now ->
        % send_to_rabbitmq({call_timer, Proc_Id0, Task_Id0}),
        io:format("tick: ~p ~p~n", [Task_Id0, Proc_Id0]),
        false;
      true -> true
    end
    end, State),
  
  {noreply, State2};


handle_info(_Request, State) ->
  {noreply, State}.


terminate(_Reason, _State) ->
  ok.

code_change(_OldVsn, State, _Extra) -> 
  {ok, State}.


